// definitions for pins, defaults, etc

#ifndef definitions_h
#define definitions_h

#include "Arduino.h"

#define SERIAL_BAUD 57600

#define MAX_SERIAL_NUMS 5
#define MAX_SERIAL_BUFF 64
#define MAX_ELECTRODE_PAIRS 240
#define MAX_LONG 2147483647

#define TIMEOUT_MS 10000
#define BLINK_RATE_TESTING 1 // blink rate while test is running; valid range: 1-100 Hz
#define BLINK_RATE_SWITCH  8 // blink rate while awaiting second switch press; valid range: 1-100 Hz
#define BLINK_TIME_SWITCH  1000 // number of milliseconds to blink LED while awaiting second switch press

#define PRESSURE_TOLERANCE 3 //pressure must be within X kPa
#define MIN_PRESSURE 1 // minimum settable pressure, kPa
#define MAX_PRESSURE 40 // minimum settable pressure, kPa
#define MAX_PRESSURE_OFFSET 5 // how much less must the max settable pressure be than the reservoir pressure
#define MIN_RESERVOIR 10
#define MAX_RESERVOIR 70
#define MIN_PROPORTIONAIR 0 // min prop-air value in hectopascals
#define MAX_PROPORTIONAIR 500 // max prop-air value in hectopascals (= 50 kPa)
#define MIN_RUNTIME 10 // shortest possible run to hold electrodes, microseconds
#define MIN_PRECON_DURATION 1
#define MAX_PRECON_DURATION 60000
#define MAX_PRECON_CYCLES 99
#define MIN_PUMP_RUNTIME 5000 // ms

#define NUMPARAMS_RUNPRECON 0
#define NUMPARAMS_RUNBIOIMP 0
#define NUMPARAMS_RUNFULL 0
#define NUMPARAMS_SETRESERVOIR 1
#define NUMPARAMS_SETPRECON 4
#define NUMPARAMS_SETELEC 1
#define NUMPARAMS_ELECSET 3
#define NUMPARAMS_ELECCOMPLETE 0
#define NUMPARAMS_SWITCHELEC 2
#define NUMPARAMS_RUNPARAMS 2
#define NUMPARAMS_SETVERB 1
#define NUMPARAMS_ABORT 0
#define NUMPARAMS_PROPORTIONAIR 1
#define NUMPARAMS_MUXHOLDHECTOPASCALS 5


#define DELIM_COM '/'
#define DELIM_NUM ','

typedef struct {
  byte onPressure;
  byte numCycles;
  unsigned int onDuration;
  unsigned int offDuration;
} preConParams_T;

typedef struct {
  unsigned long holdus;
  byte probePressure;
} bioRunParams_T;

typedef struct {
  byte Vsupply;
  char Pmax;
  char Pmin;
  byte Pin;
} pressureSensor_T;
// Honeywell's equation for output: Vout = 0.8Vsupply * (Pport - Pmin) / (Pmax - Pmin) + 0.1*Vsupply
// Pport(kPa) = (Vout - 0.1*Vsupply)*(Pmax-Pmin)/(0.8*Vsupply)+Pmin

typedef enum {
  DEBUG_NONE,
  DEBUG_SOME,
  DEBUG_MOST,
  DEBUG_ALL
} debug_e;

typedef enum {
  PIN_RX =0,
  PIN_TX, //1
  PIN_MUX1_B1, //2
  PIN_MUX1_B2, //3
  PIN_MUX1_B3, //4
  PIN_MUX1_B4, //5
  PIN_MUX2_B1, //6
  PIN_MUX2_B2, //7
  PIN_MUX2_B3, //8
  PIN_MUX2_B4, //9
  PIN_PROP_WRITE, //10
  PIN_PUMP_POWER, //11
  PIN_SOLENOID_POWER, //12
  PIN_MUX_ENABLE, //13
  PIN_PROBE_SWITCH, //14
  PIN_PROBE_LED_RED, //15
  PIN_PROBE_LED_GREEN, //16
  PIN_PROBE_PRESSURE, //17
  PIN_BOX_LED,
  PIN_PUMP_PRESSURE,
  PIN_PROP_READ,
  PIN_MUX_TRIGGER
}PINS_e;

#define PINOFFSET_FIRST PIN_MUX1_B1
#define PINOFFSET_LAST PIN_MUX_TRIGGER
#define PINOFFSET_MUX1 PIN_MUX1_B1
#define PINOFFSET_MUX2 PIN_MUX2_B1

#endif


