﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BHComBus.ArduinoComm
{
    public sealed class ArduinoMessage 
    {
        
        

        public MessageType Type { get; private set; }
        public string ArgString { get; private set; }
        public string MessageString { get; private set; }

        private ArduinoMessage()
        {

        }

        private ArduinoMessage(MessageType msgType, string rest, string original)
        {
            Type = msgType;
            ArgString = rest;
            MessageString = original;
        }

        public static ArduinoMessage Parse(string msg)
        {
            msg = msg == null ? String.Empty : msg;
            
            var m = new ArduinoMessage(MessageType.ARDUINO_UNKNOWN, String.Empty, msg);
            if (msg.Length == 0) return m;
            switch (msg[0])
            {
                case 'A':
                    if(msg.StartsWith("ArduinoReady")) m.Type = MessageType.ARDUINO_NOTIFY_READY;
                    return m;
                case 'E':
                    if (msg.StartsWith("ERROR")) m.Type = MessageType.ARDUINO_ERROR;
                    return m;
                case 'D':
                    if (msg.StartsWith("DEBUG")) m.Type = MessageType.ARDUINO_DEBUG;
                    return m;
                case 'p':
                    if (msg.StartsWith("preconditioningSkipped")) m.Type = MessageType.ARDUINO_NOTIFY_PRECON_SKIPPED;
                    else if (msg.StartsWith("preconditioningComplete")) m.Type = MessageType.ARDUINO_NOTIFY_PRECON_COMPLETE;
                    else if (msg.StartsWith("pressuresCalibrated")) m.Type = MessageType.ARDUINO_NOTIFY_PRESSURES_CALIBRATED;
                    else if (msg.StartsWith("preconParamsReceived")) m.Type = MessageType.ARDUINO_CONFIRM_PRECON_CONFIGURED;
                    else if (msg.StartsWith("paramsReceived")) m.Type = MessageType.ARDUINO_CONFIRM_BIOIMPEDANCE_PARAMS_RECEIVED;
                    return m;
                case 'b':
                    if (msg.StartsWith("bioimpedanceComplete")) m.Type = MessageType.ARDUINO_NOTIFY_BIOIMPEDANCE_COMPLETE;
                    return m;
                case 'v':
                    if (msg.StartsWith("verbosityConfirmed")) m.Type = MessageType.ARDUINO_CONFIRM_VERBOSITY;
                    return m;
                case 'e':
                    if (msg.StartsWith("electrodePairsReceived")) m.Type = MessageType.ARDUINO_NOTIFY_ELECTRODE_PAIRS_RECEIVED;
                    else if(msg.StartsWith("electrodePairsFailure")) m.Type = MessageType.ARDUINO_REQUEST_ELECTRODE_PAIR_RESEND;
                    return m;
                case 'l':
                    if(msg.StartsWith("ledTestComplete")) m.Type = MessageType.ARDUINO_NOTIFY_LED_TEST_COMPLETE;
                    return m;
                case 'r':
                    if(msg.StartsWith("requestBioParams")) m.Type = MessageType.ARDUINO_REQUEST_BIOPARAMS;
                    return m;
                case 'R':
                    if(msg.StartsWith("RETRY:")) m.Type = MessageType.ARDUINO_REQUEST_RETRY;
                    return m;
                case 's':
                    if (msg.StartsWith("startSending")) m.Type = MessageType.ARDUINO_REQUEST_ELECTRODE_PAIR;
                    else if (msg.StartsWith("setReceived")) m.Type = MessageType.ARDUINO_CONFIRM_ELECTRODE_PAIR_RECEIVED;
                    else if (msg.StartsWith("setPropComplete")) m.Type = MessageType.ARDUINO_NOTIFY_SET_PROPORTION_COMPLETE;
                    else if (msg.StartsWith("stop")) m.Type = MessageType.ARDUINO_NOTIFY_STOP;
                    return m;
                case 'm':
                    if (msg.StartsWith("muxHoldComplete")) m.Type = MessageType.ARDUINO_NOTIFY_MUX_HOLD_COMPLETE;
                    return m;
               //Add one sentence here.
          
                default:
                    return m;


            }
        }


    }

    public sealed class PCMessage
    {
        public MessageType Type { get; private set; }
        public string MessageString { get; private set; }

        public static PCMessage ABORT() 
        { return new PCMessage(MessageType.PC_REQUEST_ABORT, "ABORT"); }
        public static PCMessage RESET() 
        { return new PCMessage(MessageType.PC_REQUEST_RESET, "RESET");  }
        public static PCMessage SET_PRECONDITION(float pressure_kPa, int cycles, int durationPressurized, int durationRelaxed)
        {
            return new PCMessage(MessageType.PC_SET_PRECONDITION,
                String.Format("setPrecondition/{0},{1},{2},{3}",
                pressure_kPa, cycles, durationPressurized, durationRelaxed));
        }
        public static PCMessage SET_ELECTRODES(int electrodeSetCount) 
        {
            return new PCMessage(MessageType.PC_SET_ELECTRODES,
                String.Format("setElectrodes/{0}", electrodeSetCount));
        }
        public static PCMessage SET_ELECTRODE_PAIR(int pairNumber, int electrode1, int electrode2)
        {
            return new PCMessage(MessageType.PC_ELECTRODE_PAIR,
                String.Format("electrodeSet/{0},{1},{2}", pairNumber, electrode1, electrode2));
        }
        public static PCMessage MUX_HOLD_HECTO_PASCALS(int hold_us, int pressure_hpa, int prop_delay_ms, int electrode1, int electrode2)
        {
            return new PCMessage(MessageType.PC_MUX_HOLD_HECTO_PASCALS,
                String.Format("muxHoldHectoPascals/{0},{1},{2},{3},{4}", hold_us, pressure_hpa,prop_delay_ms, electrode1, electrode2));
        }
        public static PCMessage ELECTRODES_COMPLETE()
        { return new PCMessage(MessageType.PC_SET_ELECTRODES_COMPLETE, "electrodePairsComplete");  }
        public static PCMessage SET_RESERVOIR(int pressure_kPa)
        { return new PCMessage(MessageType.PC_SET_RESERVOIR, String.Format("setReservoir/{0}", pressure_kPa)); }
        public static PCMessage SET_PROP_HPA(int pressure_hPa)
        { return new PCMessage(MessageType.PC_SET_PROPORTION_HPA, String.Format("setPropHectoPascals/{0}", pressure_hPa)); }
        public static PCMessage RUN_PRECONDITIONING()
        { return new PCMessage(MessageType.PC_RUN_PRECONDITIONING, "runPreconditioning"); }
        public static PCMessage RUN_BIOIMPEDANCE()
        { return new PCMessage(MessageType.PC_RUN_BIOIMPEDANCE, "runBioimpedance"); }
        public static PCMessage RUN_READY()
        { return new PCMessage(MessageType.PC_RUN_READY, "runReady"); }
        public static PCMessage RUN_PARAMS(int holdTime_us, float runPressure_kPa)
        { return new PCMessage(MessageType.PC_RUN_PARAMS, String.Format("runParams/{0},{1}", holdTime_us, runPressure_kPa)); }
        public static PCMessage BIOIMPEDANCE_COMPLETE()
        { return new PCMessage(MessageType.PC_BIOIMPEDANCE_COMPLETE, "bioimpedanceComplete"); }
        public static PCMessage RUN_FULL_CYCLE()
        { return new PCMessage(MessageType.PC_RUN_FULL, "runFullCycle"); }
        public static PCMessage SET_VERBOSITY(int verbosityLevel)
        { return new PCMessage(MessageType.PC_SET_VERBOSITY, String.Format("setVerbosity/{0}", verbosityLevel));  }
        public static PCMessage PCReady()
        { return new PCMessage(MessageType.PC_READY, "PCReady"); }
        public static PCMessage CHECK_ATMO_CAL()
        { return new PCMessage(MessageType.PC_CHECK_ATMO_CAL, "checkAtmoCal"); }

        private PCMessage(MessageType type, string msg) 
        {
            Type = type;
            MessageString = msg;
        }

        public static implicit operator string(PCMessage msg) { return msg.ToString(); }

        public override string ToString()
        {
            return MessageString;
        }
    }

    public enum MessageType {

        #region Arduino messages
        ARDUINO_ERROR,
        ARDUINO_DEBUG,
        ARDUINO_NOTIFY_ABORT,
        ARDUINO_REQUEST_RETRY,
        ARDUINO_NOTIFY_READY,
        ARDUINO_CONFIRM_PRECON_CONFIGURED,
        ARDUINO_CONFIRM_BIOIMPEDANCE_PARAMS_RECEIVED,
        ARDUINO_NOTIFY_PRECON_SKIPPED,
        ARDUINO_NOTIFY_PRECON_COMPLETE,
        ARDUINO_NOTIFY_BIOIMPEDANCE_COMPLETE,
        ARDUINO_CONFIRM_VERBOSITY,
        ARDUINO_NOTIFY_PRESSURES_CALIBRATED,
        ARDUINO_NOTIFY_ELECTRODE_PAIRS_RECEIVED,
        ARDUINO_REQUEST_ELECTRODE_PAIR,
        ARDUINO_CONFIRM_ELECTRODE_PAIR_RECEIVED,
        ARDUINO_REQUEST_ELECTRODE_PAIR_RESEND,
        ARDUINO_NOTIFY_LED_TEST_COMPLETE,
        ARDUINO_REQUEST_BIOPARAMS,
        ARDUINO_BOOL_RESPONSE,
        ARDUINO_UNKNOWN,
        ARDUINO_NOTIFY_SET_PROPORTION_COMPLETE,
        ARDUINO_NOTIFY_MUX_HOLD_COMPLETE,
        //ADD ONE HERE
       ARDUINO_NOTIFY_STOP,
        #endregion

        #region PC messages
        PC_REQUEST_ABORT,
        PC_REQUEST_RESET,
        PC_SET_PRECONDITION,
        PC_SET_ELECTRODES,
        PC_ELECTRODE_PAIR,
        PC_SET_ELECTRODES_COMPLETE,
        PC_SET_RESERVOIR,
        PC_SET_PROPORTION_HPA,
        PC_RUN_PRECONDITIONING,
        PC_RUN_BIOIMPEDANCE,
        PC_RUN_PARAMS,
        PC_RUN_FULL,
        PC_RUN_READY,
        PC_BIOIMPEDANCE_COMPLETE,
        PC_SET_VERBOSITY,
        PC_READY,
        PC_CHECK_ATMO_CAL,
        PC_MUX_HOLD_HECTO_PASCALS
        #endregion

    }
}
