﻿#define DEBUG

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Diagnostics;
using BHComBus.Data;
using BHComBus.Util;


namespace BHComBus
{

    partial class MainForm
    {
        private List<ElectrodeListBoxAdapter> E1ElectrodeListBoxAdapters, E2ElectrodeListBoxAdapters;
        private BindingList<PressureAdapter> PressureList = new BindingList<PressureAdapter>();
        private BindingList<SignalAdapter> SignalList = new BindingList<SignalAdapter>();

        private float _preconPressure;
        public float PreconPressure {
            get { return _preconPressure; }
            set { 
                _preconPressure = value;
                InvokePropertyChanged("PreconPressure");
                updateRecipe();
            }
        }

        private int _preconCycles;
        public int PreconCycles
        {
            get { return _preconCycles; }
            set { _preconCycles = value; InvokePropertyChanged("PreconCycles"); updateRecipe(); }
        }

        private int _preconHoldTime;
        public int PreconHoldTime
        {
            get { return _preconHoldTime; }
            set { _preconHoldTime = value; InvokePropertyChanged("PreconHoldTime"); updateRecipe(); }
        }

        private int _preconRelaxTime;
        public int PreconRelaxTime
        {
            get { return _preconRelaxTime; }
            set { _preconRelaxTime = value; InvokePropertyChanged("PreconRelaxTime"); updateRecipe(); }
        }


        private ProtocolManager _protocolManager = new ProtocolManager();


        public bool CanRunBioimpedance
        {
            get
            {
                return true;   
            }
        }

        private void initAdHocExperiment(int electrodeCount)
        {
            #region Setup preconditioning viewmodel
            _preconPressure = 5;
            _preconCycles = 1;
            _preconHoldTime = 1000;
            _preconRelaxTime = 1000;

            PreconPressureTextbox.DataBindings.Add("Text", this, "PreconPressure", true, DataSourceUpdateMode.OnPropertyChanged);
            PreconCycleCountTextbox.DataBindings.Add("Text", this, "PreconCycles", true, DataSourceUpdateMode.OnPropertyChanged);
            PreconHoldTimeTextbox.DataBindings.Add("Text", this, "PreconHoldTime", true, DataSourceUpdateMode.OnPropertyChanged);
            PreconRelaxTimeTextbox.DataBindings.Add("Text", this, "PreconRelaxTime", true, DataSourceUpdateMode.OnPropertyChanged);
          
            #endregion

            #region Setup run settings viewmodel
            
            

            #endregion

            #region Old Code
            /*
            #region Setup Signal List viewmodel
            SignalList.Add(new SignalAdapter(10000, 10));
            SignalList.Add(new SignalAdapter(15000, 10));

            SignalList.AllowNew = true;

            SignalDataGridView.DataSource = SignalList;

            SignalDataGridView.CellValidating += SignalDataGridView_CellValidating;
            SignalDataGridView.CellEndEdit += SignalDataGridView_CellEndEdit;
            SignalDataGridView.CellValueChanged += SignalDataGridView_CellValueChanged;
            #endregion

            #region Setup Pressure List viewmodel
            PressureList.Add(new PressureAdapter(30));
            PressureList.Add(new PressureAdapter(35));

            PressureList.AllowNew = true;

            PressureListDataGridView.DataSource = PressureList;
            
            PressureListDataGridView.CellValidating +=PressureListDataGridView_CellValidating;
            PressureListDataGridView.CellEndEdit += PressureListDataGridView_CellEndEdit;
            PressureListDataGridView.CellValueChanged += PressureListDataGridView_CellValueChanged;
            #endregion

            #region Setup Electrode List viewmodel
            E1ElectrodeListBoxAdapters = new List<ElectrodeListBoxAdapter>();
            E2ElectrodeListBoxAdapters = new List<ElectrodeListBoxAdapter>();

            for (int i = 0; i < electrodeCount; i++)
            {
                E1ElectrodeListBoxAdapters.Add(new ElectrodeListBoxAdapter(this, i));
                
                E2ElectrodeListBoxAdapters.Add(new ElectrodeListBoxAdapter(this, i));
            }

            ((ListBox)E1CheckedListBox).DataSource = E1ElectrodeListBoxAdapters;
            ((ListBox)E2CheckedListBox).DataSource = E2ElectrodeListBoxAdapters;

            ((ListBox)E1CheckedListBox).DisplayMember = "ElectrodeNumber";
            ((ListBox)E2CheckedListBox).DisplayMember = "ElectrodeNumber";

            ((ListBox)E1CheckedListBox).ValueMember = "IsSelected";
            ((ListBox)E1CheckedListBox).ValueMember = "IsSelected";

            E1CheckedListBox.ItemCheck += E1CheckedListBox_ItemCheck;
            E2CheckedListBox.ItemCheck += E2CheckedListBox_ItemCheck;
            #endregion
            */
            #endregion
        }

        private void RunBioimpedanceButton_Click(object sender, EventArgs e)
        {
            updateRecipe();
            if (_protocolManager != null)
            {
                _protocolManager.run();
            }
        }



        private void frequencyComboBox_SelectedValueChanged(object sender, EventArgs e)
        {
            updateRecipe();
        }

        private void pressureStartComboBox_SelectedValueChanged(object sender, EventArgs e)
        {
            updateRecipe();
        }

        private void pressureEndComboBox_SelectedValueChanged(object sender, EventArgs e)
        {
            updateRecipe();
        }

        private void ProbeTipConfigComboBox_SelectedValueChanged(object sender, EventArgs e)
        {
            updateRecipe();
        }

        void SignalDataGridView_CellValueChanged(object sender, DataGridViewCellEventArgs e)
        {
            updateRecipe();
        }

        void PressureListDataGridView_CellValueChanged(object sender, DataGridViewCellEventArgs e)
        {
            updateRecipe();
        }

/*
        void SignalDataGridView_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            SignalDataGridView.Rows[e.RowIndex].ErrorText = String.Empty;
        }

        void SignalDataGridView_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {
            if (SignalDataGridView.Columns[e.ColumnIndex].Name == "Frequency")
            {
                int freq;
                if (!Int32.TryParse(e.FormattedValue.ToString(), out freq) || freq < BH_CONSTANTS.MIN_FREQ || freq > BH_CONSTANTS.MAX_FREQ)
                {
                    e.Cancel = true;
                    SignalDataGridView.Rows[e.RowIndex].ErrorText = "Frequency values must be between " + BH_CONSTANTS.MIN_FREQ + " and " + BH_CONSTANTS.MAX_FREQ + ".";
                }
            }
            else if(SignalDataGridView.Columns[e.ColumnIndex].Name == "SampleTime")
            {
                int per;
                if (!Int32.TryParse(e.FormattedValue.ToString(), out per) || per < BH_CONSTANTS.MIN_SAMPLE_TIME || per > BH_CONSTANTS.MAX_SAMPLE_TIME)
                {
                    e.Cancel = true;
                    SignalDataGridView.Rows[e.RowIndex].ErrorText = "The sample time must be between " + BH_CONSTANTS.MIN_SAMPLE_TIME + " and " + BH_CONSTANTS.MAX_SAMPLE_TIME + ".";
                }
            }
            
        }

        private void PressureListDataGridView_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {
            float cellvalue;
            if (!Single.TryParse(e.FormattedValue.ToString(), out cellvalue) || cellvalue < BH_CONSTANTS.MIN_PRESSURE || cellvalue > BH_CONSTANTS.MAX_PRESSURE)
            {
                e.Cancel = true;
                PressureListDataGridView.Rows[e.RowIndex].ErrorText = "Pressure values must be between " + BH_CONSTANTS.MIN_PRESSURE + " and " + BH_CONSTANTS.MAX_PRESSURE + " (kPa).";
            }
        }

        private void PressureListDataGridView_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            PressureListDataGridView.Rows[e.RowIndex].ErrorText = String.Empty;
        }
*/
        private void E2CheckedListBox_ItemCheck(object sender, ItemCheckEventArgs e)
        {
           E2ElectrodeListBoxAdapters[e.Index].IsSelected = (e.NewValue == CheckState.Checked);
#if DEBUG
            debugElectrodes();
#endif
            updateRecipe();
        }

        void E1CheckedListBox_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            E1ElectrodeListBoxAdapters[e.Index].IsSelected = (e.NewValue == CheckState.Checked);
#if DEBUG
            debugElectrodes();
#endif
            updateRecipe();
        }

        private void updateRecipe()
        {
            var recipe = new ExperimentRecipe();
            recipe.ElectrodePairsList = new List<ElectrodePair>();
            recipe.preConditioningConfiguration = new ConditioningConfig();
            recipe.SamplingParametersList = new List<SamplingParameters>();

            var pre = recipe.preConditioningConfiguration;
            pre.Pressure_kPa = PreconPressure;
            pre.CycleCount = PreconCycles;
            pre.TimeOn_ms = PreconHoldTime;
            pre.TimeOff_ms = PreconRelaxTime;

            string freqList = frequencyComboBox.Text;
            string[] freqs = freqList.Split(',').Select(sValue => sValue.Trim()).ToArray();
            recipe.FrequencyHzList = new List<int>();
            foreach (string freq in freqs) {
                recipe.FrequencyHzList.Add(Int32.Parse(freq));
            }

            string pressureList = pressureStartComboBox.Text;
            string[] pressures = pressureList.Split(',').Select(sValue => sValue.Trim()).ToArray();
            recipe.PressuresHpaList = new List<int>();
            foreach (string pressure in pressures)
            {
                recipe.PressuresHpaList.Add(Int32.Parse(pressure)*10);
            }
         

            if (ProbeTipConfigComboBox.Text == "Helical")
            {
                recipe.ElectrodePairsList = BHProbeLib.ProbeTipConfiguation.helicalConfiguration().horizontalElectrodePairOrder;
            }
            else
            {
                recipe.ElectrodePairsList = BHProbeLib.ProbeTipConfiguation.linearConfiguration().horizontalElectrodePairOrder;
            }
            Debug.WriteLine("Updated Recipe");
            if(_protocolManager != null)
                _protocolManager.Recipe = recipe;
        }

        private Queue<BackgroundWorker> _debugWorkers = new Queue<BackgroundWorker>();
        private void debugElectrodes()
        {
            
            var _debugPrinterWorker = new BackgroundWorker();
            _debugPrinterWorker.DoWork += _debugPrinterWorker_DoWork;
            _debugWorkers.Enqueue(_debugPrinterWorker);
            _debugPrinterWorker.RunWorkerAsync();
        }

        void _debugPrinterWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            Debug.WriteLine("Current Electrode Combination:");
            Debug.Indent();
            foreach (var e1 in (from e1 in E1ElectrodeListBoxAdapters where e1.IsSelected select e1))
                foreach (var e2 in (from e2 in E2ElectrodeListBoxAdapters where e2.IsSelected select e2))
                    Debug.WriteLine("E1[" + e1.ElectrodeNumber + "] X E2[" + e2.ElectrodeNumber + "]");
            Debug.Unindent();
            _debugWorkers.Dequeue();
            
        }
       
    }

    public class SignalAdapter
    {
        public int Frequency { get; set; }
        public int SampleTime { get; set; }

        public SignalAdapter(int freq, int p)
        {
            Frequency = freq;
            SampleTime = p;
        }

        public SignalAdapter()
        {
            Frequency = 10000;
            SampleTime = 1000;
        }
    }

    public class PressureAdapter
    {
        private float _pressure;
        public float Pressure
        {
            get { return _pressure; }
            set { _pressure = value; }
        }
        public PressureAdapter(float P)
        {
            _pressure = P;
        }

        public PressureAdapter()
        {
            _pressure = BH_CONSTANTS.MIN_PRESSURE;
        }
    }

    public class ElectrodeListBoxAdapter : INotifyPropertyChanged
    {

        public int ElectrodeNumber { get; set; }
        private bool _isSelected;
        public bool IsSelected {
            get { return _isSelected; }
            set {
                _isSelected = value;
                InvokePropertyChanged("IsSelected");
            }
        }

        private Form _parentForm;
        public ElectrodeListBoxAdapter(Form parentForm, int electrodeNumber)
        {
            ElectrodeNumber = electrodeNumber;
            IsSelected = false;
            _parentForm = parentForm;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void InvokePropertyChanged(string property) {
            if (_parentForm != null && _parentForm.IsHandleCreated && _parentForm.InvokeRequired)
                _parentForm.Invoke(new Action(() => {
                    var _pC = PropertyChanged;
                    if(_pC != null)
                        _pC(this, new PropertyChangedEventArgs(property));
                }));
            else
            {
                var _pC = PropertyChanged;
                if (_pC != null)
                    _pC(this, new PropertyChangedEventArgs(property));
            }
                
        }
    }
}