﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using System.Diagnostics;
using System.Linq;
using BHComBus.Util;
using BHComBus.FGU;
using BHComBus.ArduinoComm;
using System.ComponentModel;


namespace BHComBus
{
    /// <summary>
    /// Description of MainForm.
    /// </summary>
    public partial class MainForm
    {
        private IEnumerable<string> COMPorts;

        private void initArduinoConfig()
        {
            COMPorts = from p in BHComBus.ArduinoComm.ArduinoNanoDeviceInterface.getCOMPortNames() select p;


            this.COMPortListSource.DataSource = COMPorts;
            string arduinoPortNum = BHComBus.ArduinoComm.ArduinoNanoDeviceInterface.getArduinoCOMPortName();
            ASCOMPortComboBox.Text = arduinoPortNum;
            Console.WriteLine("Arduino on " + arduinoPortNum);
            ASCOMPortComboBox_SelectedIndexChanged(null, null);
        }


        private void ASRefreshButton_Click(object sender, EventArgs e)
        {
            initArduinoConfig();
        }

        private void comboBox1_SelectionChangeCommitted(object sender, EventArgs e)
        {
            /*
            State.ArduinoNanoDeviceInterface oldDevice = State.ArduinoNanoDeviceInterface.currentDevice;
            State.ArduinoNanoDeviceInterface.SetDevice(comboBox1.SelectedItem.ToString());
            if(oldDevice != State.ArduinoNanoDeviceInterface.currentDevice)
                State.ArduinoNanoDeviceInterface.currentDevice.OnReceiveSerialData += currentDevice_OnReceiveSerialData;
             * */
        }

        void currentDevice_OnReceiveSerialData(BHComBus.ArduinoComm.IArduinoDeviceInterface device, string data)
        {

            if (data.Trim().Length == 0) return;


        }

        private void ASSubmitButton_Click(object sender, EventArgs e)
        {
            if (ASSubmitTextBox.Text.Length > 0)
            {
                if (BHComBus.ArduinoComm.ArduinoNanoDeviceInterface.currentDevice != null)
                {
                    BHComBus.ArduinoComm.ArduinoNanoDeviceInterface.currentDevice.sendString(ASSubmitTextBox.Text + "\n");
                    ASSubmitTextBox.Text = "";
                }
            }
        }

        private void ASSubmitTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)13)
            {
                if (ASSubmitTextBox.Text.Length > 0)
                {
                    if (BHComBus.ArduinoComm.ArduinoNanoDeviceInterface.currentDevice != null)
                    {
                        BHComBus.ArduinoComm.ArduinoNanoDeviceInterface.currentDevice.sendString(ASSubmitTextBox.Text + "\n");
                        ASSubmitTextBox.Text = "";
                    }
                }
            }
        }

        private void groupBox2_Enter(object sender, EventArgs e)
        {

        }

        private void ASCOMPortComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            var selectedItem = ASCOMPortComboBox.SelectedItem;
            if (selectedItem == null) return;
            string selectedPort = selectedItem.ToString();
            if (!selectedPort.StartsWith("COM")) return;

            ArduinoNanoDeviceInterface oldDevice = ArduinoNanoDeviceInterface.currentDevice;
            try
            {
                ArduinoNanoDeviceInterface.SetDevice(selectedPort);
            }
            catch (Exception ex)
            {

            }
            if (oldDevice != ArduinoNanoDeviceInterface.currentDevice)
            {
                
                var cd = ArduinoNanoDeviceInterface.currentDevice;
                cd.OnReceiveSerialData += currentDevice_OnReceiveSerialData;
                cd.OnSerialPortWrite += currentDevice_OnSerialPortWrite;
                cd.OnReceiveMessage += currentDevice_OnReceiveMessage;
                cd.OnReceiveErrorMessage += currentDevice_OnReceiveErrorMessage;
                cd.OnReceiveDebugMessage += currentDevice_OnReceiveDebugMessage;

                if (oldDevice != null)
                {
                    oldDevice.OnArduinoStatusChange -= currentDevice_OnArduinoStatusChange;
                }
                cd.OnArduinoStatusChange += currentDevice_OnArduinoStatusChange;
                CurrentArduinoDeviceStatus = cd.Status;
                _protocolManager.Arduino = cd;
            }
        }

        private ArduinoStatus _currentArduinoDeviceStatus;
        public ArduinoStatus CurrentArduinoDeviceStatus
        {
            get { return _currentArduinoDeviceStatus;  }
            set
            {
                _currentArduinoDeviceStatus = value;
                InvokePropertyChanged("CurrentArduinoDeviceStatus");
            }
        }
        void currentDevice_OnArduinoStatusChange(IArduinoDeviceInterface device, ArduinoStatus oldStatus, ArduinoStatus newStatus)
        {
            CurrentArduinoDeviceStatus = newStatus;
        }

        void currentDevice_OnReceiveDebugMessage(BHComBus.ArduinoComm.IArduinoDeviceInterface device, string data)
        {
            ASRecvTextBox.Invoke((Action)delegate
            {
                DEBUGTextBox.Text += "=> " + data.Trim() + Environment.NewLine;
                DEBUGTextBox.SelectionStart = DEBUGTextBox.Text.Length - 2;
                DEBUGTextBox.ScrollToCaret();
            });
        }

        void currentDevice_OnReceiveErrorMessage(BHComBus.ArduinoComm.IArduinoDeviceInterface device, string data)
        {
            ASRecvTextBox.Invoke((Action)delegate
            {
                ERRORTextBox.Text += "=> " + data.Trim() + Environment.NewLine;
                ERRORTextBox.SelectionStart = ERRORTextBox.Text.Length - 2;
                ERRORTextBox.ScrollToCaret();
            });
        }

        void currentDevice_OnReceiveMessage(BHComBus.ArduinoComm.IArduinoDeviceInterface device, string data)
        {
            ASRecvTextBox.Invoke((Action)delegate
            {
                ASRecvTextBox.Text += "=> " + data.Trim() + Environment.NewLine;
                ASRecvTextBox.SelectionStart = ASRecvTextBox.Text.Length - 2;
                ASRecvTextBox.ScrollToCaret();
            });
        }

        int cmdIdx = 0;

        void currentDevice_OnSerialPortWrite(BHComBus.ArduinoComm.IArduinoDeviceInterface device, string data)
        {
            ASRecvTextBox.Invoke((Action)delegate
            {
                string txt = "[[" + data.Trim() + "]] === [" + cmdIdx + "] ===" + Environment.NewLine;
                ASRecvTextBox.Text += txt;
                ERRORTextBox.Text += txt;
                DEBUGTextBox.Text += txt;
                cmdIdx++;
                DEBUGTextBox.SelectionStart = DEBUGTextBox.Text.Length - 2;
                DEBUGTextBox.ScrollToCaret();
                ERRORTextBox.SelectionStart = ERRORTextBox.Text.Length - 2;
                ERRORTextBox.ScrollToCaret();
                ASRecvTextBox.SelectionStart = ASRecvTextBox.Text.Length - 2;
                ASRecvTextBox.ScrollToCaret();
            });
        }

    }
}