﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using System.Diagnostics;
using System.Linq;
using BHComBus.Util;
using BHComBus.FGU;
using BHComBus.ArduinoComm;
using BHComBus.Data;
using System.ComponentModel;
using System.Threading;
using System.IO.Ports;
using MsgBox;

namespace BHComBus
{
    /// <summary>
    /// Description of MainForm.
    /// </summary>
    public partial class MainForm
    {

        private GenStatus _FGUStatus;
        public GenStatus FGUStatus
        {
            get { return _FGUStatus; }
            set
            {
                _FGUStatus = value;
                InvokePropertyChanged("FGUStatus");
            }
        }

        private Thread _FGThread;


        private void initFG()
        {
            COMUtils.FGPort = new SerialPort();
            COMUtils.FGPort.BaudRate = 57600;
            COMUtils.FGPort.Parity = Parity.None;
            COMUtils.FGPort.StopBits = StopBits.One;
            COMUtils.FGPort.DataBits = 8;
            COMUtils.FGPort.Handshake = Handshake.None;
            COMUtils.FGPort.DataReceived += new SerialDataReceivedEventHandler(ProtocolManager.p_DataReceived);
            COMUtils.FGPortNum = null;
            foreach (COMPort port in COMUtils.GetComPorts())
            {
                if (port.HardwareName.Contains("Prolific"))
                {
                    COMUtils.FGPortNum = port.portNum;
                }
            }
            if (COMUtils.FGPortNum == null)
            {
                Console.WriteLine("FG Not Found! Requesting user selection");
                bool selected = false;
                List<COMPort> portList;
                InputBox.SetLanguage(InputBox.Language.English);
                while (!selected)
                {
                    portList = COMUtils.GetComPorts();
                    string[] portStrList = new string[portList.Count];
                    int i = 0;
                    foreach (COMPort port in portList)
                    {
                        portStrList[i++] = port.portNum;
                    }
                
                    //Save the DialogResult as res
                    DialogResult res = InputBox.ShowDialog(
                    "Could not find the Function Generator automatically. Please select the correct port, refresh, or press Cancel to exit.",
                    "Select FG Port",   //Text message (mandatory), Title (optional)
                        InputBox.Icon.Nothing, //Set icon type (default info)
                        InputBox.Buttons.YesNoCancel, //Set buttons (default ok)
                        InputBox.Type.ComboBox, //Set type (default nothing)
                        portStrList, //String field as ComboBox items (default null)
                        false); //Set visible in taskbar (default false)

                    //Check InputBox result
                    if (res == System.Windows.Forms.DialogResult.Cancel)
                    {
                        Environment.Exit(-1);
                    }
                    if (res == System.Windows.Forms.DialogResult.Yes)
                    {
                        COMUtils.FGPortNum = InputBox.ResultValue; //Get returned value
                        selected = true;
                        try
                        {
                            COMUtils.FGPort.PortName = COMUtils.FGPortNum;
                            COMUtils.FGPort.Open();
                        }
                        catch (System.UnauthorizedAccessException ex)
                        {
                            selected = false;
                            MessageBox.Show("Error in connecting to the selected COM port. Try again");
                        }
                    }
                }
                
            }
            else {

                Console.WriteLine("Function Generator on " + COMUtils.FGPortNum);
                COMUtils.FGPort.PortName = COMUtils.FGPortNum;
                COMUtils.FGPort.Open();
            }
            FunctionGeneratorStatusTextbox.Text = COMUtils.FGPortNum;
            
        }

        public FunctionGeneratorSettings FGSettings
        {
            get { return new FunctionGeneratorSettings(_curFunc, _curFreq, _curAmp); }
            private set { }
        }

        private int _curFreq;
        private int _newFreq;
        private int _curAmp;
        private int _newAmp;
        private Function _curFunc;
        private Function _newFunc;

        private bool _showGen;
        private void FGLoop()
        {
            //FGUImports.ShowGen(FGUICheckBox.Checked);
            FGUImports.OpenGen();
            while (true)
            {
                if (_curFreq != _newFreq || _curAmp != _newAmp || _curFunc != _newFunc)
                {
                    _curFreq = _newFreq;
                    _curAmp = _newAmp;
                    _curFunc = _newFunc;
                    FGUImports.SetGen(_newFunc, _newFreq, _newAmp, 0);
                    Thread.Sleep(1500);
                    InvokePropertyChanged("FunctionGeneratorSettings");
                }
                FGUStatus = FGUImports.GenStatus();
                Thread.Sleep(200);
            }
        }

        private void FGUICheckBox_CheckStateChanged(object sender, EventArgs e)
        {
            //FGUImports.ShowGen(FGUICheckBox.Checked);
        }

        private DateTime lastClickTime = DateTime.Now;
        private void FGStartButton_Click(object sender, EventArgs e)
        {
            // give the function generator control program 2 secs to start up.
            // Once the control program has started, additional clicks do nothing bad
            if (lastClickTime.AddSeconds(2) > DateTime.Now) return;

            lastClickTime = DateTime.Now;
            //FGUImports.ShowGen(FGUICheckBox.Checked);
            FGUImports.OpenGen();
            

        }
    }
}