﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace BHComBus.Device
{

    #region Generic Device Interfaces
    public enum DeviceStatus
    {
        NOT_CONNECTED,
        READY,
        BUSY,
        ERROR
    }

    public delegate void OscilloscopeStatusListener(IOscilloscope device, DeviceStatus status);
    public delegate void FunctionGeneratorStatusListener(IFunctionGenerator device, DeviceStatus status);
    public delegate void PicoScopeDriverStatusListener(PicoScope.PicoStatus newStatus);

    public interface IOscilloscope
    {
        DeviceStatus OscilloscopeStatus { get; }
        event OscilloscopeStatusListener OnOscilloscopeStatusChange;
    }

    public interface IFunctionGenerator
    {
        DeviceStatus FunctionGeneratorStatus { get; }
        event FunctionGeneratorStatusListener OnFunctionGeneratorStatusChange;

        void setFrequency(int freq_hz);
        void setAmplitude(int voltage_mv);
    }
    #endregion

    public class PicoscopeDeviceManager : IOscilloscope, INotifyPropertyChanged
    {
        #region INotifyPropertyChanged Implementation

        public event PropertyChangedEventHandler PropertyChanged;

        public void InvokePropertyChanged(PropertyChangedEventArgs e)
        {
            var handler = PropertyChanged;
            if (handler != null) handler(this, e);
        }

        public void InvokePropertyChanged(string property)
        {
            InvokePropertyChanged(new PropertyChangedEventArgs(property));
        }
        
        #endregion

        public event OscilloscopeStatusListener OnOscilloscopeStatusChange;
        public event FunctionGeneratorStatusListener OnFunctionGeneratorStatusChange;
        public event PicoScopeDriverStatusListener OnPicoScopeDriverStatusChange;
        public static bool scopeready;
        private DeviceStatus _deviceStatus;
        public DeviceStatus OscilloscopeStatus
        {
            get { return _deviceStatus; }
            private set
            {
                _deviceStatus = value;
                InvokePropertyChanged("OscilloscopeStatus");
            }
        }

        private PicoScope.PicoStatus _picoDriverStatus;
        private PicoScope.PicoStatus PicoDriverStatus
        {
            get { return _picoDriverStatus; }
            set
            {
                _picoDriverStatus = value;
                InvokePropertyChanged("PicoDriverStatus");
            }
        }

        public static PicoscopeDeviceManager GetDeviceManager()
        {
            return new PicoscopeDeviceManager();
        }

        private Thread _deviceManagerThread;
        private short _picoScopeDriverHandle;

        private PicoscopeDeviceManager()
        {
            OscilloscopeStatus = DeviceStatus.NOT_CONNECTED;
            _picoScopeDriverHandle = 0;
            deviceInit();
        }

        public void deviceInit()
        {
            if (Thread.CurrentThread != _deviceManagerThread)
            {
                _deviceManagerThread = new Thread(deviceInit);
                _deviceManagerThread.IsBackground = true;
                _deviceManagerThread.Start();
                return;
            }
            var openResult = PicoScope.OpenUnit(out _picoScopeDriverHandle);
            if (_picoScopeDriverHandle > 0) { 
                PicoDriverStatus = PicoScope.PicoStatus.PICO_OK;
                OscilloscopeStatus = DeviceStatus.READY;
                
                string[] description = {
                           "Driver Version    ",
                           "USB Version       ",
                           "Hardware Version  ",
                           "Variant Info      ",
                           "Serial            ",
                           "Cal Date          ",
                           "Kernel Ver        "
                                       };
            System.Text.StringBuilder line = new System.Text.StringBuilder(80);

            int variant;
            for (int i = 0; i < 7; i++)
            {
                short requiredSize;
                PicoScope.GetUnitInfo(_picoScopeDriverHandle, line, 80, out requiredSize, i);
                if (i == 3)
                {
                    line.Length = 4;
                    variant = Convert.ToInt16(line.ToString());
                }
                Console.WriteLine("{0}: {1}", description[i], line);
            }
            scopeready = true;
                Thread.Sleep(100);
                var flashStatus = PicoScope.FlashLed(_picoScopeDriverHandle, (short)-15);
            }
            if (Thread.CurrentThread.IsBackground)
                Thread.CurrentThread.Join();
        }

    }


}
