﻿using System.Collections.Generic;
using System.Management;
using System.Text.RegularExpressions;
using System.Text;
using System;
using Microsoft.Win32;
using System.IO.Ports;
using System.Runtime.InteropServices;

namespace BHComBus.Util
{

   
    public  class Formulae
    {
        public int getSampleTimeInMicroSecs(int freqInHz, int periodCount)
        {
            int samplePeriodFreq = freqInHz / periodCount;
            return 1000000 / samplePeriodFreq;
        }
    }

    public class COMPort
    {
        /*public string HardwareID { get; set; }
        public string PortName { get; set; }
        public string FriendlyName { get; set; }*/
        public string HardwareName { get; set; }
        public string portNum { get; set; }
    }

    public static class COMUtils
    {
        //Gets a list of COM ports by checking likely registry keys for PortName properties with a value that starts
        //with COM (e.g., "COM4", "COM6", etc.)
        public static string FGPortNum;
        public static SerialPort FGPort;
        public static List<COMPort> GetComPorts()
        {
            
            List<COMPort> comports = new List<COMPort>();

            RegistryKey rk1 = Registry.LocalMachine;
            RegistryKey rk2 = rk1.OpenSubKey("HARDWARE\\DEVICEMAP\\SERIALCOMM");
            foreach (String portName in rk2.GetValueNames())
            {
                COMPort curPort = new COMPort();
                curPort.HardwareName = portName;
                curPort.portNum = (string)rk2.GetValue(portName);
                comports.Add(curPort);
            }
            
            return comports;
        }
    }

    //Allocates a new pinned array, or pins an existing array.  Allows DMA and other interproc shared memory
    //operations with C DLLs etc. that do not run on the .NET VM
    public class PinnedArray<T> : IDisposable
    {
        private GCHandle _GCHandle;
        private bool _wasFreed;

        public PinnedArray(int arraySize) : this(new T[arraySize]) { }

        public PinnedArray(T[] array)
        {
            _GCHandle = GCHandle.Alloc(array, GCHandleType.Pinned);
        }

        ~PinnedArray()
        {
            Dispose(false);
        }

        public T[] Target
        {
            get { return (T[])_GCHandle.Target; }
        }

        public static implicit operator T[](PinnedArray<T> a)
        {
            if (a == null)
                return null;

            return (T[])a._GCHandle.Target;
        }

        public void Dispose()
        {
            Dispose(true);

            //The dispose call above frees the memory explicitly, so finalization is not necessary
            GC.SuppressFinalize(this);
        }

        private void Dispose(bool disposing)
        {
            if (_wasFreed)
                return;

            _wasFreed = true;

            if (disposing)
            {
                // placeholder in case any IDisposable members are added to this class.  If so, dispose here.
                
            }

            _GCHandle.Free();
        }
    }
}