/*********************************************************************
 *
 *  PIC32 Function Generator Code
 *  Bruce Land
 ********************************************************************/
// all peripheral library includes
#include <plib.h>
#include <math.h>

// Configuration Bit settings
// SYSCLK = 40 MHz (8MHz Crystal/ FPLLIDIV * FPLLMUL / FPLLODIV)
// PBCLK = 40 MHz
// Primary Osc w/PLL (XT+,HS+,EC+PLL)
// WDT OFF
// Other options are don't care
//
#pragma config FNOSC = FRCPLL, POSCMOD = HS, FPLLIDIV = DIV_2, FPLLMUL = MUL_15, FPBDIV = DIV_1, FPLLODIV = DIV_1
#pragma config FWDTEN = OFF
// turn off alternative functions for port pins B.4 and B.5
#pragma config JTAGEN = OFF, DEBUG = OFF
#pragma config FSOSCEN = OFF

// frequency we're running at
#define	SYS_FREQ 60000000

// UART parameters
#define BAUDRATE 57600// must match PC end
#define PB_DIVISOR (1 << OSCCONbits.PBDIV) // read the peripheral bus divider, FPBDIV
#define PB_FREQ SYS_FREQ/PB_DIVISOR // periperhal bus frequency

// useful ASCII/VT100 macros for PuTTY
#define clrscr() printf( "\x1b[2J")
#define home()   printf( "\x1b[H")
#define pcr()    printf( '\r')
#define crlf     putchar(0x0a); putchar(0x0d);
#define backspace 0x08
#define max_chars 10 // for input buffer

// receive function prototype (see below for code)
//int GetDataBuffer(char *buffer, int max_size);
void GetDataBuffer(char *buffer, int max_size);
//***************************************************
#define str_buffer_size 20
char str_buffer[str_buffer_size];

// the sine lookup table
volatile unsigned char sine_table[256];

// main ////////////////////////////
int main(void)
{
        int sine_table_size ;
        int timer_limit ;
        unsigned char s;
        //#define F_OUT 80000
        int F_OUT = 10000 ;
        // sine table index
        int i;

         __XC_UART = 2;

    /* 	Initialize PPS */
    // Data sheet:
    // http://people.ece.cornell.edu/land/courses/ece4760/PIC32/Microchip_stuff/2xx_datasheet.pdf
    // data sheet table 11-1 gives input pin mapping
    // data sheet table 11.2 gives output pin mapping
    // Also Section 12 of PIC32MX Family Reference Manual
    // http://hades.mech.northwestern.edu/images/2/21/61132B_PIC32ReferenceManual.pdf
    // macro defs
    // http://quadcopter-miami-ece.googlecode.com/svn/trunk/mpide-0023-windows-20120903/hardware/pic32/compiler/pic32-tools/pic32mx/include/peripheral/pps.h
    //
    // specify PPS group, signal, logical pin name
    PPSInput (2, U2RX, RPB11); //Assign U2RX to pin RPB11 -- Physical pin 22 on 28 PDIP
    PPSOutput(4, RPB10, U2TX); //Assign U2TX to pin RPB10 -- Physical pin 21 on 28 PDIP

    ANSELA =0; //make sure analog is cleared
    ANSELB =0;

    // init the uart2
    UARTConfigure(UART2, UART_ENABLE_PINS_TX_RX_ONLY);
    UARTSetLineControl(UART2, UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1);
    UARTSetDataRate(UART2, PB_FREQ, BAUDRATE);
    UARTEnable(UART2, UART_ENABLE_FLAGS(UART_PERIPHERAL | UART_RX | UART_TX));

        // calibrate the internal clock
	SYSKEY = 0x0; // ensure OSCCON is locked
        SYSKEY = 0xAA996655; // Write Key1 to SYSKEY
        SYSKEY = 0x556699AA; // Write Key2 to SYSKEY
        // OSCCON is now unlocked
        // make the desired change
        OSCTUN = 31; // tune the rc oscillator
        // Relock the SYSKEY
        SYSKEY = 0x0; // Write any value other than Key1 or Key2
        // OSCCON is relocked

        // Configure the device for maximum performance but do not change the PBDIV
	// Given the options, this function will change the flash wait states, RAM
	// wait state and enable prefetch cache but will not change the PBDIV.
	// The PBDIV value is already set via the pragma FPBDIV option above..
	SYSTEMConfig(SYS_FREQ, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);

        // frequency settable to better than 2% relative accuracy at low frequency
        // frequency settable to 5% accuracy at highest frequency
        // Useful frequency range 10 Hz to 100KHz
        // >40 db attenuation of  next highest amplitude frequency component below 100 kHz
        // >35 db above 100 KHz




        // Set up timer2 on,  interrupts, internal clock, prescalar 1, toggle rate
        // Uses macro to set timer tick to 2 microSec = 120 cycles 500 kHz DDS
        // peripheral at 60 MHz
        //--------------------
        //------------------
        // 64 LEVEL samples thru external DAC
        // interval=60, 32 samples, 31200 Hz, next harmonic 40 db down
        // interval=20, 32 samples, 90.8 KHz, next harmonic 45 db down
        // interval=600, 32 samples, 3170 Hz, next harmonic 32 db down
        // interval=300, 64 samples, 3170 Hz, next harmonic 38 db down
        // interval=150, 128 samples, 3170 Hz, next harmonic 43  db down


        // Open the desired DMA channel.
        #define dmaChn 0
	// We enable the AUTO option, we'll keep repeating the sam transfer over and over.
	DmaChnOpen(dmaChn, 0, DMA_OPEN_AUTO);

	// set the transfer event control: what event is to start the DMA transfer
        // In this case, timer2
	DmaChnSetEventControl(dmaChn, DMA_EV_START_IRQ(_TIMER_2_IRQ));

	// once we configured the DMA channel we can enable it
	// now it's ready and waiting for an event to occur...
	DmaChnEnable(dmaChn);

        // set up i/o port pin
        ANSELA =0; // turn off analog on A
        mPORTAClearBits(BIT_0);		//Clear A bits to ensure light is off.
        mPORTASetPinsDigitalOut(BIT_0);    //Set A port as output
        PORTSetBits(IOPORT_A, BIT_0);
        // set up DAC pins
        ANSELB =0; // turn off analog on B
        // See also pragmas to turn off alternative functions
        mPORTBClearBits(BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5);
        mPORTBSetPinsDigitalOut(BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_7 );    //Set port B as output
        i=0;
        clrscr();  //clear PuTTY screen

	while(1)
	{
           // toggle a bit for perfromance measure
            //mPORTBToggleBits(BIT_7);
            printf(">");
            printf("Please type a number for the frequncy\n\r");
            //sscanf("%d", &F_OUT);
            GetDataBuffer(str_buffer, max_chars);
            // check to see if a valid number was typed and use it
           // float temp;
            if (sscanf(str_buffer, "%d", &F_OUT)) {
                // printf("The frequnecy you entered is %f \n\r", temp);
                F_OUT=1000*F_OUT;
               printf("The frequnecy you entered is %d \n\r", F_OUT);
                if (F_OUT <= 5000 ){
                    sine_table_size = 256 ;
                    timer_limit = SYS_FREQ/(sine_table_size*F_OUT) ;
                }
                else if (F_OUT <= 10000 ){
                    sine_table_size = 128 ;
                    timer_limit = SYS_FREQ/(sine_table_size*F_OUT) ;
                }
                else if (F_OUT <= 20000 ){
                    sine_table_size = 64 ;
                    timer_limit = SYS_FREQ/(sine_table_size*F_OUT) ;
                }
                else if (F_OUT <= 100000 ){
                    sine_table_size = 32 ;
                    timer_limit = SYS_FREQ/(sine_table_size*F_OUT) ;
                }
                else {
                    sine_table_size = 16 ;
                    timer_limit = SYS_FREQ/(sine_table_size*F_OUT) ;
                }
                // build the sine lookup table
                // scaled to produce values between 0 and 63 (six bit)

                for (i = 0; i < sine_table_size; i++){
                //sine_table[i] =(unsigned char) (31.5 * sin((float)i*6.283/(float)sine_table_size) + 32); //
                   // s =(unsigned char) (63.5 * sin((float)i*6.283/(float)sine_table_size) + 64); //7 bit
                    s =(unsigned char) (63.5 * sin((float)i*6.283/(float)sine_table_size) + 64); //7 bit
                    sine_table[i] = (s & 0x3f) | ((s & 0x40)<<1) ;
                 }
                // set up timer
                 OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_1, timer_limit);
                 // set up DMA
                 // set the transfer parameters: source & destination address, source & destination size, number of bytes per event
                // Setting the last parameter to one makes the DMA output one byte/interrupt
                DmaChnSetTxfer(dmaChn, sine_table, (void*)&LATB, sine_table_size, 1, 1);


            }

 	}
	return 0;
}


/////////////////////////////////////////////////////////////////
// build a string from the UART2 /////////////
//////////////////////////////////////////////
//int GetDataBuffer(char *buffer, int max_size)
void GetDataBuffer(char *buffer, int max_size)

{
    int num_char;
    num_char = 0;
    memset(buffer, 0, max_size);

    while(num_char < max_size)
    {
        char character;

        // get the character
        while(!UARTReceivedDataIsAvailable(UART2)){};
        character = UARTGetDataByte(UART2);
        while(!UARTTransmitterIsReady(UART2)){};
        UARTSendDataByte(UART2, character);

        // end line
        if(character == '\r'){
            *buffer = 0; // zero terminate the string
            //crlf; // send a new line
            break;
        }
        // backspace
        if (character == backspace){
            putchar(' '); // write a blank over the previous character
            putchar(backspace); // go back one position
            buffer--;
            num_char--;
        }
        else { // save it all
            *buffer = character;
            buffer++;
            num_char++;
        } //if (character == backspace)

    } //while(num_char < max_size)
    //return num_char;
}





